/*
	3D Buttons Demo
	
	Public Domain source by Zig Zichterman
	ziggr@aol.com
	
	A real quick and dirty demo for the 3D Buttons CDEF.
	
	If you put CDEF 0 in the resource file ("3D Buttons Demo..rsrc") for
	this project, the demo will use it for the dialog. I have already 
	put version 1.0b1 of the 3D Buttons CDEF in the resource file.
	
	06/29/94	zz	 these comments	
	
*/

void	InitializeToolbox();
void	Toggle(DialogPtr inDialog, short inItemNum);
void	Tristate(DialogPtr inDialog, short inItemNum);
void	Disable(DialogPtr inDialog, short inItemNum);
void	SetValue(DialogPtr inDialog, short inItemNum, short inValue);

enum {
	push_enabled	= 1,
	check_enabled,
	radio1_enabled,
	radio2_enabled,
	icon1_enabled,
	icon2_enabled,
	
	push_disabled,
	check_disabled,
	radio1_disabled,
	radio2_disabled,
	icon1_disabled,
	icon2_disabled,
	
	push_trireset,
	
	check_tristate,
	radio1_tristate,
	radio2_tristate,
	
	exit
};

void
main(void)
{
	InitializeToolbox();
	MaxApplZone();
	
	DialogPtr	dialog = ::GetNewDialog(128,NULL,(WindowPtr) -1);
	if (!dialog) return;
	
	// set up dialog
	SetValue(dialog,radio1_enabled,1);
	SetValue(dialog,check_tristate,2);
	SetValue(dialog,radio1_tristate,2);
	SetValue(dialog,radio2_tristate,2);
	SetValue(dialog,radio1_disabled,1);
	
	Disable(dialog,push_disabled);
	Disable(dialog,check_disabled);
	Disable(dialog,radio1_disabled);
	Disable(dialog,radio2_disabled);
	Disable(dialog,icon1_disabled);
	Disable(dialog,icon2_disabled);

	SetDialogDefaultItem(dialog,exit);
	
	short itemHit;
	do {
		::ModalDialog(NULL,&itemHit);
		
		// handle clicks
		switch (itemHit) {
			case check_enabled  :
				Toggle(dialog,itemHit);
				break;
			case radio1_enabled  :
			case radio1_tristate :
				SetValue(dialog,itemHit,1);
				SetValue(dialog,itemHit + 1,0);
				break;
			case radio2_enabled  :
			case radio2_tristate :
				SetValue(dialog,itemHit,1);
				SetValue(dialog,itemHit - 1,0);
				break;
			case check_tristate :
				Tristate(dialog,itemHit);
				break;
			case push_trireset  :
				SetValue(dialog,check_tristate,2);
				SetValue(dialog,radio1_tristate,2);
				SetValue(dialog,radio2_tristate,2);
				break;
		}
	} while (itemHit != exit);
	
	::DisposeDialog(dialog);
}

/**************************************************************************
	Toggle()
	
	Toggle the value of a control item in a dialog
**************************************************************************/
void
Toggle(DialogPtr inDialog, short inItemNum)
{
	short	iType;
	Handle	iHandle;
	Rect	iRect;
	short	iValue;
	
	GetDItem(inDialog,inItemNum,&iType,&iHandle,&iRect);
	if (iHandle) {
		iValue = GetCtlValue((ControlHandle) iHandle);
		iValue = !iValue;
		SetCtlValue((ControlHandle) iHandle,iValue);
	}
}

/**************************************************************************
	Disable()
	
	Disable (dim) a control item in a dialog
**************************************************************************/
void
Disable(DialogPtr inDialog, short inItemNum)
{
	short	iType;
	Handle	iHandle;
	Rect	iRect;
	short	iValue;
	
	GetDItem(inDialog,inItemNum,&iType,&iHandle,&iRect);
	iType |= itemDisable;
	SetDItem(inDialog,inItemNum,iType,iHandle,&iRect);
	
	if (iType & ctrlItem) {
		HiliteControl((ControlHandle) iHandle,255);
	}
}

/**************************************************************************
	Tristate()
	
	Rotate a control value through 0,1 and 2 (2 = tristate)
**************************************************************************/
void
Tristate(DialogPtr inDialog, short inItemNum)
{
	short	iType;
	Handle	iHandle;
	Rect	iRect;
	short	iValue;
	
	GetDItem(inDialog,inItemNum,&iType,&iHandle,&iRect);
	if (iHandle) {
		iValue = GetCtlValue((ControlHandle) iHandle);
		if (++iValue > 2) iValue = 0;
		SetCtlValue((ControlHandle) iHandle,iValue);
	}
}

/**************************************************************************
	SetValue()
	
	Set the value of a control
**************************************************************************/
void
SetValue(DialogPtr inDialog, short inItemNum, short inValue)
{
	short	iType;
	Handle	iHandle;
	Rect	iRect;
	
	GetDItem(inDialog,inItemNum,&iType,&iHandle,&iRect);
	if (iHandle) {
		short max = GetCtlMax((ControlHandle) iHandle);
		if (max < inValue) SetCtlMax((ControlHandle) iHandle, inValue);
		SetCtlValue((ControlHandle) iHandle,inValue);
	}
}


/**************************************************************************
	InitializeToolbox()
	
	The requisite toolbox initialization that every application under
	the sun has.
**************************************************************************/
void
InitializeToolbox()
{
	InitGraf((Ptr) &qd.thePort);		// Toolbox Managers
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
}

